<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads
 *
 * @package   jchoptimize/joomla-platform
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2020 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 * If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */
namespace JchOptimize\Model;

\defined('_JEXEC') or die('Restricted Access');
use FilesystemIterator;
use JchOptimize\Core\PageCache\CaptureCache;
use JchOptimize\Platform\Paths;
use Joomla\CMS\Factory;
use Joomla\DI\ContainerAwareInterface;
use Joomla\DI\ContainerAwareTrait;
use Joomla\Event\Dispatcher;
use Joomla\Filesystem\Folder;
use _JchOptimizeVendor\Laminas\Cache\Exception\ExceptionInterface;
use _JchOptimizeVendor\Laminas\Cache\Storage\Adapter\Filesystem;
use _JchOptimizeVendor\Laminas\Cache\Storage\FlushableInterface;
use _JchOptimizeVendor\Laminas\Cache\Storage\IterableInterface;
use _JchOptimizeVendor\Laminas\Cache\Storage\StorageInterface;
use _JchOptimizeVendor\Laminas\Cache\Storage\TaggableInterface;
use RecursiveDirectoryIterator;
use RecursiveIteratorIterator;
class Cache implements ContainerAwareInterface
{
    use ContainerAwareTrait;
    /**
     * @var StorageInterface
     */
    private $cache;
    public function __construct(StorageInterface $cache)
    {
        $this->cache = $cache;
    }
    public function getCacheSize() : array
    {
        $size = 0;
        $numFiles = 0;
        if ($this->cache instanceof IterableInterface) {
            try {
                $iterator = $this->cache->getIterator();
                $numFiles += \iterator_count($iterator);
                foreach ($iterator as $item) {
                    //Let's skip the 'test' cache set on instantiation in container
                    if ($item == \md5('__ITEM__')) {
                        $numFiles -= 1;
                        continue;
                    }
                    $metaData = $this->cache->getMetadata($item);
                    if (!\is_array($metaData)) {
                        continue;
                    }
                    if (isset($metaData['size'])) {
                        $size += $metaData['size'];
                    } else {
                        if (\file_exists($metaData['filespec'] . '.dat')) {
                            $size += \filesize($metaData['filespec'] . '.dat');
                        }
                    }
                }
            } catch (ExceptionInterface|\Exception $e) {
            }
        }
        //Iterate through the static files
        if (\file_exists(Paths::cachePath(\false))) {
            $directory = new RecursiveDirectoryIterator(Paths::cachePath(\false), FilesystemIterator::SKIP_DOTS);
            $iterator = new RecursiveIteratorIterator($directory);
            $i = 0;
            foreach ($iterator as $file) {
                if ($file->getFilename() == 'index.html') {
                    $i++;
                    continue;
                }
                $size += $file->getSize();
            }
            $numFiles += \iterator_count($iterator) - $i;
        }
        $decimals = 2;
        $sz = 'BKMGTP';
        $factor = (int) \floor((\strlen($size) - 1) / 3);
        $size = \sprintf("%.{$decimals}f", $size / \pow(1024, $factor)) . $sz[$factor];
        $numFiles = \number_format($numFiles);
        return [$size, $numFiles];
    }
    /**
     * Cleans cache from the server
     *
     * @return bool
     */
    public function cleanCache() : bool
    {
        $success = \false;
        //Clean all cache generated by Storage
        if ($this->cache instanceof FlushableInterface) {
            $success |= $this->cache->flush();
        }
        if (!$this->cache instanceof IterableInterface || !$this->cache instanceof TaggableInterface) {
            //Filesystem cache was used for tagging so let's flush those
            /** @var Filesystem $fsCache */
            $fsCache = $this->container->get(Filesystem::class);
            $fsCache->flush();
        }
        if (JCH_PRO) {
            //Perhaps Capture Cache was used, so we'll need to clean the htaccess files
            /** @var CaptureCache $captureCache */
            $captureCache = $this->container->get(CaptureCache::class);
            $captureCache->deleteAllItems();
        }
        //Delete any static combined files
        $staticCachePath = Paths::cachePath(\false);
        try {
            if (\file_exists($staticCachePath . '/css')) {
                $success |= Folder::delete($staticCachePath . '/css');
            }
            if (\file_exists($staticCachePath . '/js')) {
                $success |= Folder::delete($staticCachePath . '/js');
            }
        } catch (\Exception $e) {
            $success |= \false;
        }
        //Delete system page cache
        $globalCache = Factory::getCache();
        $cacheGroups = ['page', 'pce'];
        foreach ($cacheGroups as $group) {
            $success |= $globalCache->clean($group);
        }
        //Clean LiteSpeed cache
        if (\file_exists(JPATH_PLUGINS . '/system/lscache/lscache.php')) {
            $dispatcher = new Dispatcher();
            $dispatcher->triggerEvent('onLSCacheExpired');
        }
        try {
            $app = Factory::getApplication();
            $app->setHeader('X-LiteSpeed-Purge', '*');
        } catch (\Exception $e) {
        }
        return $success;
    }
}
